// LICENSE : MIT
"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _FileCache = require("./FileCache");

var _FileCache2 = _interopRequireDefault(_FileCache);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var md5 = require("md5");
var fs = require("fs");
var path = require("path");
var os = require("os");

var defaultOptions = {
    cacheDir: os.tmpdir(),
    // default: 5 minutes
    maxAge: 5 * 60 * 1000
};

var FileCacheManager = function () {
    function FileCacheManager() {
        var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

        _classCallCheck(this, FileCacheManager);

        this.cacheDir = options.cacheDir || defaultOptions.cacheDir;
        this.cacheFilePath = path.join(this.cacheDir, ".file-cache-cli.json");
        this.maxAge = options.maxAge !== undefined ? options.maxAge : defaultOptions.maxAge;
    }

    /**
     * @returns {FileCache[]}
     */


    _createClass(FileCacheManager, [{
        key: "createFileCache",


        /**
         * @param {string} filePath
         * @returns {FileCache}
         */
        value: function createFileCache(filePath) {
            var content = fs.readFileSync(filePath, "utf-8");
            return new _FileCache2.default({
                filePath: filePath,
                hash: md5(content),
                lastChecked: Date.now()
            });
        }

        /**
         * filter `filePathList` by cached data
         * @param {string[]} filePathList
         * @returns {string[]} filtered filePathList
         */

    }, {
        key: "filterByCache",
        value: function filterByCache() {
            var _this = this;

            var filePathList = arguments.length <= 0 || arguments[0] === undefined ? [] : arguments[0];

            var fileCaches = this.fileCaches;
            var targetFileCacheList = filePathList.filter(function (filePath) {
                return filePath.length > 0;
            }).filter(function (filePath) {
                return fs.lstatSync(filePath).isFile();
            }).map(function (filePath) {
                return _this.createFileCache(path.resolve(process.cwd(), filePath));
            });
            var currentTime = Date.now();
            var filteredFilePathList = targetFileCacheList.filter(function (targetFile) {
                return !fileCaches.some(function (fileCache) {
                    // true, should be ignored file
                    if (fileCache.equals(targetFile)) {
                        return true;
                    }
                    // old data should be checked
                    if (fileCache.lastChecked < currentTime - _this.maxAge) {
                        return true;
                    }
                    return false;
                });
            });
            // save
            this.fileCaches = targetFileCacheList;
            return filteredFilePathList.map(function (fileCache) {
                return fileCache.filePath;
            });
        }

        /**
         * clean cache
         */

    }, {
        key: "clean",
        value: function clean() {
            try {
                fs.unlinkSync(this.cacheFilePath);
            } catch (error) {}
        }
    }, {
        key: "fileCaches",
        get: function get() {
            try {
                var json = require(this.cacheFilePath);
                return json.map(function (object) {
                    return new _FileCache2.default(object);
                });
            } catch (error) {
                return [];
            }
        }

        /**
         * Update cache data
         * @param {FileCache[]} data
         */
        ,
        set: function set(data) {
            fs.writeFileSync(this.cacheFilePath, JSON.stringify(data, null, 4));
        }
    }]);

    return FileCacheManager;
}();

exports.default = FileCacheManager;
//# sourceMappingURL=FileCacheManager.js.map